package com.ruoyi.system.service.impl;

import java.util.Collection;
import java.util.Iterator;
import com.ruoyi.common.core.exception.ServiceException;
import java.util.List;
import com.ruoyi.common.core.utils.StringUtils;
import com.ruoyi.common.core.text.Convert;
import com.ruoyi.system.domain.SysConfig;
import javax.annotation.PostConstruct;
import com.ruoyi.common.redis.service.RedisService;
import org.springframework.beans.factory.annotation.Autowired;
import com.ruoyi.system.mapper.SysConfigMapper;
import org.springframework.stereotype.Service;
import com.ruoyi.system.service.ISysConfigService;

@Service
public class SysConfigServiceImpl implements ISysConfigService
{
    @Autowired
    private SysConfigMapper configMapper;
    @Autowired
    private RedisService redisService;
    
    @PostConstruct
    public void init() {
        this.loadingConfigCache();
    }
    
    public SysConfig selectConfigById(final Long configId) {
        final SysConfig config = new SysConfig();
        config.setConfigId(configId);
        return this.configMapper.selectConfig(config);
    }
    
    public String selectConfigByKey(final String configKey) {
        final String configValue = Convert.toStr(this.redisService.getCacheObject(this.getCacheKey(configKey)));
        if (StringUtils.isNotEmpty(configValue)) {
            return configValue;
        }
        final SysConfig config = new SysConfig();
        config.setConfigKey(configKey);
        final SysConfig retConfig = this.configMapper.selectConfig(config);
        if (StringUtils.isNotNull((Object)retConfig)) {
            this.redisService.setCacheObject(this.getCacheKey(configKey), (Object)retConfig.getConfigValue());
            return retConfig.getConfigValue();
        }
        return "";
    }
    
    public SysConfig selectConfigByKeyObj(final String configKey) {
        final SysConfig config = new SysConfig();
        config.setConfigKey(configKey);
        final SysConfig retConfig = this.configMapper.selectConfig(config);
        if (StringUtils.isNotNull((Object)retConfig)) {
            this.redisService.setCacheObject(this.getCacheKey(configKey), (Object)retConfig.getConfigValue());
            return retConfig;
        }
        return null;
    }
    
    public List<SysConfig> selectConfigList(final SysConfig config) {
        return this.configMapper.selectConfigList(config);
    }
    
    public int insertConfig(final SysConfig config) {
        final int row = this.configMapper.insertConfig(config);
        if (row > 0) {
            this.redisService.setCacheObject(this.getCacheKey(config.getConfigKey()), (Object)config.getConfigValue());
        }
        return row;
    }
    
    public int updateConfig(final SysConfig config) {
        final SysConfig temp = this.configMapper.selectConfigById(config.getConfigId());
        if (!StringUtils.equals((CharSequence)temp.getConfigKey(), (CharSequence)config.getConfigKey())) {
            this.redisService.deleteObject(this.getCacheKey(temp.getConfigKey()));
        }
        final int row = this.configMapper.updateConfig(config);
        if (row > 0) {
            this.redisService.setCacheObject(this.getCacheKey(config.getConfigKey()), (Object)config.getConfigValue());
        }
        return row;
    }
    
    public void deleteConfigByIds(final Long[] configIds) {
        for (final Long configId : configIds) {
            final SysConfig config = this.selectConfigById(configId);
            if (StringUtils.equals((CharSequence)"Y", (CharSequence)config.getConfigType())) {
                throw new ServiceException(String.format("内置参数【%1$s】不能删除 ", config.getConfigKey()));
            }
            this.configMapper.deleteConfigById(configId);
            this.redisService.deleteObject(this.getCacheKey(config.getConfigKey()));
        }
    }
    
    public void loadingConfigCache() {
        final List<SysConfig> configsList = this.configMapper.selectConfigList(new SysConfig());
        for (final SysConfig config : configsList) {
            this.redisService.setCacheObject(this.getCacheKey(config.getConfigKey()), (Object)config.getConfigValue());
        }
    }
    
    public void clearConfigCache() {
        final Collection<String> keys = this.redisService.keys("sys_config:*");
        this.redisService.deleteObject((Collection)keys);
    }
    
    public void resetConfigCache() {
        this.clearConfigCache();
        this.loadingConfigCache();
    }
    
    public boolean checkConfigKeyUnique(final SysConfig config) {
        final Long configId = StringUtils.isNull((Object)config.getConfigId()) ? -1L : config.getConfigId();
        final SysConfig info = this.configMapper.checkConfigKeyUnique(config.getConfigKey());
        return !StringUtils.isNotNull((Object)info) || info.getConfigId() == (long)configId;
    }
    
    private String getCacheKey(final String configKey) {
        return "sys_config:" + configKey;
    }
}
